@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitbuild.c,v 1.107 2005/04/07 21:07:18 nickm Exp $ */
-const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.107 2005/04/07 21:07:18 nickm Exp $";
+/* $Id: circuitbuild.c,v 1.109 2005/04/26 18:52:15 arma Exp $ */
+const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.109 2005/04/26 18:52:15 arma Exp $";
 
 /**
  * \file circuitbuild.c
@@ -346,7 +346,8 @@ int circuit_handle_first_hop(circuit_t *circ) {
 void circuit_n_conn_done(connection_t *or_conn, int status) {
   circuit_t *circ;
 
-  log_fn(LOG_DEBUG,"or_conn to %s, status=%d", or_conn->nickname, status);
+  log_fn(LOG_DEBUG,"or_conn to %s, status=%d",
+         or_conn->nickname ? or_conn->nickname : "NULL", status);
 
   for (circ=global_circuitlist;circ;circ = circ->next) {
     if (circ->marked_for_close)
@@ -819,9 +820,7 @@ static int new_route_len(double cw, uint8_t purpose, smartlist_t *routers) {
     routelen = 4;
   else {
     log_fn(LOG_WARN,"Bug: unhandled purpose %d", purpose);
-#ifdef TOR_FRAGILE
-    tor_assert(0);
-#endif
+    tor_fragile_assert();
     return -1;
   }
 #endif
@@ -1137,9 +1136,7 @@ choose_good_exit_server(uint8_t purpose, routerlist_t *dir,
       return r;
   }
   log_fn(LOG_WARN,"Bug: unhandled purpose %d", purpose);
-#ifdef TOR_FRAGILE
-  tor_assert(0);
-#endif
+  tor_fragile_assert();
   return NULL;
 }
 
