@@ -1,9 +1,9 @@
 /* Copyright 2001 Matej Pfajfar.
  * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitlist.c,v 1.34 2005/03/29 00:35:35 arma Exp $ */
-const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.34 2005/03/29 00:35:35 arma Exp $";
+/* $Id: circuitlist.c,v 1.45 2005/04/08 03:26:23 nickm Exp $ */
+const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.45 2005/04/08 03:26:23 nickm Exp $";
 
 /**
  * \file circuitlist.c
@@ -11,26 +11,103 @@ const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.34 2005/03/29 00:35:35 a
  **/
 
 #include "or.h"
+#include "tree.h"
 
 /********* START VARIABLES **********/
 
 /** A global list of all circuits at this hop. */
 circuit_t *global_circuitlist=NULL;
 
-/** Array of strings to make circ-\>state human-readable */
-const char *circuit_state_to_string[] = {
-  "doing handshakes",        /* 0 */
-  "processing the onion",    /* 1 */
-  "connecting to firsthop",  /* 2 */
-  "open"                     /* 3 */
-};
-
-/********* END VARIABLES ************/
-
 static void circuit_free(circuit_t *circ);
 static void circuit_free_cpath(crypt_path_t *cpath);
 static void circuit_free_cpath_node(crypt_path_t *victim);
 
+/********* END VARIABLES ************/
+
+/** DOCDOC This whole section */
+struct orconn_circid_circuit_map_t {
+  RB_ENTRY(orconn_circid_circuit_map_t) node;
+  connection_t *or_conn;
+  uint16_t circ_id;
+  circuit_t *circuit;
+};
+
+static INLINE int compare_orconn_circid_entries(
+       struct orconn_circid_circuit_map_t *a,
+       struct orconn_circid_circuit_map_t *b)
+{
+  if (a->or_conn < b->or_conn)
+    return -1;
+  else if (a->or_conn > b->or_conn)
+    return 1;
+  else
+    return ((int)b->circ_id) - ((int)a->circ_id);
+};
+static RB_HEAD(orconn_circid_tree, orconn_circid_circuit_map_t) orconn_circid_circuit_map = RB_INITIALIZER(orconn_circid_circuit_map);
+RB_PROTOTYPE(orconn_circid_tree, orconn_circid_circuit_map_t, node, compare_orconn_circid_entries);
+RB_GENERATE(orconn_circid_tree, orconn_circid_circuit_map_t, node, compare_orconn_circid_entries);
+
+struct orconn_circid_circuit_map_t *_last_circid_orconn_ent = NULL;
+
+void
+circuit_set_circid_orconn(circuit_t *circ, uint16_t id,
+                          connection_t *conn,
+                          enum which_conn_changed_t which)
+{
+  uint16_t old_id;
+  connection_t *old_conn;
+  struct orconn_circid_circuit_map_t search;
+  struct orconn_circid_circuit_map_t *found;
+
+  tor_assert(!conn || conn->type == CONN_TYPE_OR);
+
+  if (which == P_CONN_CHANGED) {
+    old_id = circ->p_circ_id;
+    old_conn = circ->p_conn;
+    circ->p_circ_id = id;
+    circ->p_conn = conn;
+  } else {
+    old_id = circ->n_circ_id;
+    old_conn = circ->n_conn;
+    circ->n_circ_id = id;
+    circ->n_conn = conn;
+  }
+
+  if (_last_circid_orconn_ent &&
+      ((old_id == _last_circid_orconn_ent->circ_id &&
+        old_conn == _last_circid_orconn_ent->or_conn) ||
+       (id == _last_circid_orconn_ent->circ_id &&
+        conn == _last_circid_orconn_ent->or_conn))) {
+    _last_circid_orconn_ent = NULL;
+  }
+
+  if (old_conn) {
+    search.circ_id = old_id;
+    search.or_conn = old_conn;
+    found = RB_FIND(orconn_circid_tree, &orconn_circid_circuit_map, &search);
+    if (found) {
+      RB_REMOVE(orconn_circid_tree, &orconn_circid_circuit_map, found);
+    }
+    tor_free(found);
+  }
+
+  if (conn == NULL)
+    return;
+
+  search.circ_id = id;
+  search.or_conn = conn;
+  found = RB_FIND(orconn_circid_tree, &orconn_circid_circuit_map, &search);
+  if (found) {
+    found->circuit = circ;
+  } else {
+    found = tor_malloc_zero(sizeof(struct orconn_circid_circuit_map_t));
+    found->circ_id = id;
+    found->or_conn = conn;
+    found->circuit = circ;
+    RB_INSERT(orconn_circid_tree, &orconn_circid_circuit_map, found);
+  }
+}
+
 /** Add <b>circ</b> to the global list of circuits. This is called only from
  * within circuit_new.
  */
@@ -71,6 +148,22 @@ void circuit_close_all_marked(void)
   }
 }
 
+/** Function to make circ-\>state human-readable */
+const char *
+circuit_state_to_string(int state) {
+  static char buf[64];
+  switch (state) {
+    case CIRCUIT_STATE_BUILDING: return "doing handshakes";
+    case CIRCUIT_STATE_ONIONSKIN_PENDING: return "processing the onion";
+    case CIRCUIT_STATE_OR_WAIT: return "connecting to firsthop";
+    case CIRCUIT_STATE_OPEN: return "open";
+    default:
+      log_fn(LOG_WARN, "Bug: unknown circuit state %d", state);
+      tor_snprintf(buf, sizeof(buf), "unknown state [%d]", state);
+      return buf;
+  }
+}
+
 /** Allocate space for a new circuit, initializing with <b>p_circ_id</b>
  * and <b>p_conn</b>. Add it to the global circuit list.
  */
@@ -84,13 +177,12 @@ circuit_t *circuit_new(uint16_t p_circ_id, connection_t *p_conn) {
 
   circ->timestamp_created = time(NULL);
 
-  circ->p_circ_id = p_circ_id;
-  circ->p_conn = p_conn;
-
   circ->state = CIRCUIT_STATE_ONIONSKIN_PENDING;
 
   /* CircIDs */
-  circ->p_circ_id = p_circ_id;
+  if (p_conn) {
+    circuit_set_circid_orconn(circ, p_circ_id, p_conn, P_CONN_CHANGED);
+  }
   /* circ->n_circ_id remains 0 because we haven't identified the next hop yet */
 
   circ->package_window = CIRCWINDOW_START;
@@ -127,6 +219,9 @@ static void circuit_free(circuit_t *circ) {
   if (circ->rend_splice) {
     circ->rend_splice->rend_splice = NULL;
   }
+  /* Remove from map. */
+  circuit_set_circid_orconn(circ, 0, NULL, P_CONN_CHANGED);
+  circuit_set_circid_orconn(circ, 0, NULL, N_CONN_CHANGED);
 
   memset(circ, 0xAA, sizeof(circuit_t)); /* poison memory */
   tor_free(circ);
@@ -208,36 +303,78 @@ circuit_get_by_global_id(uint32_t id)
  *    in p_streams or n_streams.
  * Return NULL if no such circuit exists.
  */
-circuit_t *circuit_get_by_circ_id_conn(uint16_t circ_id, connection_t *conn) {
+circuit_t *circuit_get_by_circid_orconn(uint16_t circ_id, connection_t *conn) {
+  struct orconn_circid_circuit_map_t search;
+  struct orconn_circid_circuit_map_t *found;
+
+  tor_assert(conn->type == CONN_TYPE_OR);
+
+  if (_last_circid_orconn_ent &&
+      circ_id == _last_circid_orconn_ent->circ_id &&
+      conn == _last_circid_orconn_ent->or_conn) {
+    found = _last_circid_orconn_ent;
+  } else {
+    search.circ_id = circ_id;
+    search.or_conn = conn;
+    found = RB_FIND(orconn_circid_tree, &orconn_circid_circuit_map, &search);
+    _last_circid_orconn_ent = found;
+  }
+  if (found && found->circuit && !found->circuit->marked_for_close)
+    return found->circuit;
+
+  /* The rest of this can be replaced with
+     "return NULL;" once we believe the code works. */
+
+  {
+    circuit_t *circ;
+    for (circ=global_circuitlist;circ;circ = circ->next) {
+      if (circ->marked_for_close)
+        continue;
+
+      if (circ->p_conn == conn && circ->p_circ_id == circ_id) {
+        log_fn(LOG_WARN, "circuit matches p_conn, but not in tree (Bug!)");
+        return circ;
+      }
+      if (circ->n_conn == conn && circ->n_circ_id == circ_id) {
+        log_fn(LOG_WARN, "circuit matches n_conn, but not in tree (Bug!)");
+        return circ;
+      }
+    }
+    return NULL;
+  }
+
+}
+
+/** DOCDOC */
+circuit_t *circuit_get_by_edge_conn(connection_t *conn)
+{
   circuit_t *circ;
   connection_t *tmpconn;
+  tor_assert(CONN_IS_EDGE(conn));
 
-  for (circ=global_circuitlist;circ;circ = circ->next) {
-    if (circ->marked_for_close)
-      continue;
-
-    if (circ->p_circ_id == circ_id) {
-      if (circ->p_conn == conn)
-        return circ;
-      for (tmpconn = circ->p_streams; tmpconn; tmpconn = tmpconn->next_stream) {
-        if (tmpconn == conn)
-          return circ;
-      }
-    }
-    if (circ->n_circ_id == circ_id) {
-      if (circ->n_conn == conn)
-        return circ;
-      for (tmpconn = circ->n_streams; tmpconn; tmpconn = tmpconn->next_stream) {
-        if (tmpconn == conn)
-          return circ;
-      }
-      for (tmpconn = circ->resolving_streams; tmpconn; tmpconn = tmpconn->next_stream) {
-        if (tmpconn == conn)
-          return circ;
-      }
+  if (! conn->on_circuit) {
+    /* return NULL; */
+    circ = circuit_get_by_conn(conn);
+    if (circ) {
+      log_fn(LOG_WARN, "BUG: conn->on_circuit==NULL, but there was in fact a circuit there. ");
     }
+    return circ;
   }
-  return NULL;
+
+  circ = conn->on_circuit;
+  /* All this stuff here is sanity-checking. */
+  tor_assert(circ->magic == CIRCUIT_MAGIC);
+  for (tmpconn = circ->p_streams; tmpconn; tmpconn=tmpconn->next_stream)
+    if (tmpconn == conn)
+      return circ;
+  for (tmpconn = circ->n_streams; tmpconn; tmpconn=tmpconn->next_stream)
+    if (tmpconn == conn)
+      return circ;
+  for (tmpconn = circ->resolving_streams; tmpconn; tmpconn=tmpconn->next_stream)
+    if (tmpconn == conn)
+      return circ;
+
+  tor_assert(0);
 }
 
 /** Return a circ such that circ is attached to <b>conn</b>, either as
@@ -389,12 +526,20 @@ void circuit_mark_all_unused_circs(void) {
  *   - If circ->rend_splice is set (we are the midpoint of a joined
  *     rendezvous stream), then mark the other circuit to close as well.
  */
-int _circuit_mark_for_close(circuit_t *circ) {
+void _circuit_mark_for_close(circuit_t *circ, int line, const char *file)
+{
   connection_t *conn;
 
   assert_circuit_ok(circ);
-  if (circ->marked_for_close)
-    return -1;
+  tor_assert(line);
+  tor_assert(file);
+
+  if (circ->marked_for_close) {
+    log(LOG_WARN,"Duplicate call to circuit_mark_for_close at %s:%d"
+        " (first at %s:%d)", file, line,
+        circ->marked_for_close_file, circ->marked_for_close);
+    return;
+  }
 
   if (circ->state == CIRCUIT_STATE_ONIONSKIN_PENDING) {
     onion_pending_remove(circ);
@@ -435,20 +580,21 @@ int _circuit_mark_for_close(circuit_t *circ) {
       conn->has_sent_end = 1; /* we're closing the circuit, nothing to send to */
       connection_mark_for_close(conn);
     }
+    conn->on_circuit = NULL;
   }
   if (circ->p_conn)
     connection_send_destroy(circ->p_circ_id, circ->p_conn);
   for (conn=circ->p_streams; conn; conn=conn->next_stream)
     connection_edge_destroy(circ->p_circ_id, conn);
 
-  circ->marked_for_close = 1;
+  circ->marked_for_close = line;
+  circ->marked_for_close_file = file;
 
   if (circ->rend_splice && !circ->rend_splice->marked_for_close) {
     /* do this after marking this circuit, to avoid infinite recursion. */
     circuit_mark_for_close(circ->rend_splice);
     circ->rend_splice = NULL;
   }
-  return 0;
 }
 
 /** Verify that cpath layer <b>cp</b> has all of its invariants
@@ -518,9 +664,14 @@ void assert_circuit_ok(const circuit_t *c)
   if (c->n_conn) {
     tor_assert(c->n_conn->type == CONN_TYPE_OR);
     tor_assert(!memcmp(c->n_conn->identity_digest, c->n_conn_id_digest, DIGEST_LEN));
+    if (c->n_circ_id)
+      tor_assert(c == circuit_get_by_circid_orconn(c->n_circ_id, c->n_conn));
   }
-  if (c->p_conn)
+  if (c->p_conn) {
     tor_assert(c->p_conn->type == CONN_TYPE_OR);
+    if (c->p_circ_id)
+      tor_assert(c == circuit_get_by_circid_orconn(c->p_circ_id, c->p_conn));
+  }
   for (conn = c->p_streams; conn; conn = conn->next_stream)
     tor_assert(conn->type == CONN_TYPE_AP);
   for (conn = c->n_streams; conn; conn = conn->next_stream)
