@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuituse.c,v 1.69 2005/04/07 21:07:19 nickm Exp $ */
-const char circuituse_c_id[] = "$Id: circuituse.c,v 1.69 2005/04/07 21:07:19 nickm Exp $";
+/* $Id: circuituse.c,v 1.70 2005/04/26 18:52:15 arma Exp $ */
+const char circuituse_c_id[] = "$Id: circuituse.c,v 1.70 2005/04/26 18:52:15 arma Exp $";
 
 /**
  * \file circuituse.c
@@ -731,9 +731,7 @@ circuit_launch_by_router(uint8_t purpose, routerinfo_t *exit,
         default:
           log_fn(LOG_WARN,"Bug: unexpected purpose %d when cannibalizing a general circ.",
                  purpose);
-#ifdef TOR_FRAGILE
-          tor_assert(0);
-#endif
+          tor_fragile_assert();
           return NULL;
       }
       return circ;
