@@ -1,9 +1,9 @@
 /* Copyright 2001 Matej Pfajfar.
  * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: command.c,v 1.81 2005/01/03 20:03:49 nickm Exp $ */
-const char command_c_id[] = "$Id: command.c,v 1.81 2005/01/03 20:03:49 nickm Exp $";
+/* $Id: command.c,v 1.84 2005/04/06 05:33:32 nickm Exp $ */
+const char command_c_id[] = "$Id: command.c,v 1.84 2005/04/06 05:33:32 nickm Exp $";
 
 /**
  * \file command.c
@@ -173,7 +173,7 @@ static void command_process_create_cell(cell_t *cell, connection_t *conn) {
     conn->circ_id_type = CIRC_ID_TYPE_HIGHER;
   }
 
-  circ = circuit_get_by_circ_id_conn(cell->circ_id, conn);
+  circ = circuit_get_by_circid_orconn(cell->circ_id, conn);
 
   if (circ) {
     log_fn(LOG_WARN,"received CREATE cell (circID %d) for known circ. Dropping.", cell->circ_id);
@@ -205,7 +205,7 @@ static void command_process_create_cell(cell_t *cell, connection_t *conn) {
 static void command_process_created_cell(cell_t *cell, connection_t *conn) {
   circuit_t *circ;
 
-  circ = circuit_get_by_circ_id_conn(cell->circ_id, conn);
+  circ = circuit_get_by_circid_orconn(cell->circ_id, conn);
 
   if (!circ) {
     log_fn(LOG_INFO,"(circID %d) unknown circ (probably got a destroy earlier). Dropping.", cell->circ_id);
@@ -245,7 +245,7 @@ static void command_process_created_cell(cell_t *cell, connection_t *conn) {
 static void command_process_relay_cell(cell_t *cell, connection_t *conn) {
   circuit_t *circ;
 
-  circ = circuit_get_by_circ_id_conn(cell->circ_id, conn);
+  circ = circuit_get_by_circid_orconn(cell->circ_id, conn);
 
   if (!circ) {
     log_fn(LOG_INFO,"unknown circuit %d on connection to %s:%d. Dropping.",
@@ -290,7 +290,7 @@ static void command_process_relay_cell(cell_t *cell, connection_t *conn) {
 static void command_process_destroy_cell(cell_t *cell, connection_t *conn) {
   circuit_t *circ;
 
-  circ = circuit_get_by_circ_id_conn(cell->circ_id, conn);
+  circ = circuit_get_by_circid_orconn(cell->circ_id, conn);
 
   if (!circ) {
     log_fn(LOG_INFO,"unknown circuit %d on connection to %s:%d. Dropping.",
@@ -305,18 +305,17 @@ static void command_process_destroy_cell(cell_t *cell, connection_t *conn) {
 
   if (cell->circ_id == circ->p_circ_id) {
     /* the destroy came from behind */
-    circ->p_conn = NULL;
+    circuit_set_circid_orconn(circ, 0, NULL, P_CONN_CHANGED);
     circuit_mark_for_close(circ);
   } else { /* the destroy came from ahead */
-    circ->n_conn = NULL;
-#if 0
-    if (!CIRCUIT_IS_ORIGIN(circ)) {
+    circuit_set_circid_orconn(circ, 0, NULL, N_CONN_CHANGED);
+    if (CIRCUIT_IS_ORIGIN(circ)) {
+      circuit_mark_for_close(circ);
+    } else {
       log_fn(LOG_DEBUG, "Delivering 'truncated' back.");
       connection_edge_send_command(NULL, circ, RELAY_COMMAND_TRUNCATED,
                                    NULL, 0, NULL);
     }
-#endif
-    circuit_mark_for_close(circ);
   }
 }
 
