@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: config.c,v 1.335 2005/04/23 19:29:09 arma Exp $ */
-const char config_c_id[] = "$Id: config.c,v 1.335 2005/04/23 19:29:09 arma Exp $";
+/* $Id: config.c,v 1.337 2005/04/27 04:13:59 arma Exp $ */
+const char config_c_id[] = "$Id: config.c,v 1.337 2005/04/27 04:13:59 arma Exp $";
 
 /**
  * \file config.c
@@ -98,7 +98,7 @@ static config_var_t config_vars[] = {
   VAR("AccountingStart",     STRING,   AccountingStart,      NULL),
   VAR("AllowUnverifiedNodes",CSV,      AllowUnverifiedNodes, "middle,rendezvous"),
   VAR("AuthoritativeDirectory",BOOL,   AuthoritativeDir,     "0"),
-  VAR("BandwidthRate",       MEMUNIT,  BandwidthRate,        "1 MB"),
+  VAR("BandwidthRate",       MEMUNIT,  BandwidthRate,        "2 MB"),
   VAR("BandwidthBurst",      MEMUNIT,  BandwidthBurst,       "5 MB"),
   VAR("MaxAdvertisedBandwidth",MEMUNIT,MaxAdvertisedBandwidth,"128 TB"),
   VAR("ClientOnly",          BOOL,     ClientOnly,           "0"),
@@ -134,6 +134,7 @@ static config_var_t config_vars[] = {
   VAR("HashedControlPassword",STRING,  HashedControlPassword, NULL),
   VAR("HttpProxy",           STRING,   HttpProxy,            NULL),
   VAR("HttpsProxy",          STRING,   HttpsProxy,           NULL),
+  VAR("HttpsProxyAuthenticator",STRING,HttpsProxyAuthenticator,NULL),
   VAR("HiddenServiceOptions",LINELIST_V, RendConfigLines,    NULL),
   VAR("HiddenServiceDir",    LINELIST_S, RendConfigLines,    NULL),
   VAR("HiddenServicePort",   LINELIST_S, RendConfigLines,    NULL),
@@ -343,8 +344,6 @@ options_act(void) {
     set_exit_redirects(sl);
   }
 
-  /* Start backgrounding the process, if requested. */
-
   /* Finish backgrounding the process */
   if (options->RunAsDaemon) {
     /* We may be calling this for the n'th time (on SIGHUP), but it's safe. */
@@ -1512,6 +1511,13 @@ options_validate(or_options_t *options)
     }
   }
 
+  if (options->HttpsProxyAuthenticator) {
+    if (strlen(options->HttpsProxyAuthenticator) >= 48) {
+      log(LOG_WARN, "HttpsProxyAuthenticator is too long (>= 48 chars).");
+      result = -1;
+    }
+  }
+
   if (options->HashedControlPassword) {
     if (decode_hashed_password(NULL, options->HashedControlPassword)<0) {
       log_fn(LOG_WARN,"Bad HashedControlPassword: wrong length or bad base64");
