@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection_edge.c,v 1.320 2005/04/07 20:25:22 nickm Exp $ */
-const char connection_edge_c_id[] = "$Id: connection_edge.c,v 1.320 2005/04/07 20:25:22 nickm Exp $";
+/* $Id: connection_edge.c,v 1.321 2005/04/26 18:52:15 arma Exp $ */
+const char connection_edge_c_id[] = "$Id: connection_edge.c,v 1.321 2005/04/26 18:52:15 arma Exp $";
 
 /**
  * \file connection_edge.c
@@ -129,9 +129,7 @@ int connection_edge_process_inbuf(connection_t *conn, int package_partial) {
       return 0;
   }
   log_fn(LOG_WARN,"Bug: Got unexpected state %d. Closing.",conn->state);
-#ifdef TOR_FRAGILE
-  tor_assert(0);
-#endif
+  tor_fragile_assert();
   connection_edge_end(conn, END_STREAM_REASON_INTERNAL, conn->cpath_layer);
   connection_mark_for_close(conn);
   return -1;
@@ -176,9 +174,7 @@ connection_edge_end(connection_t *conn, char reason, crypt_path_t *cpath_layer)
 
   if (conn->has_sent_end) {
     log_fn(LOG_WARN,"Harmless bug: Calling connection_edge_end (reason %d) on an already ended stream?", reason);
-#ifdef TOR_FRAGILE
-    tor_assert(0);
-#endif
+    tor_fragile_assert();
     return -1;
   }
 
@@ -251,9 +247,7 @@ int connection_edge_finished_flushing(connection_t *conn) {
       return 0;
     default:
       log_fn(LOG_WARN,"BUG: called in unexpected state %d.", conn->state);
-#ifdef TOR_FRAGILE
-      tor_assert(0);
-#endif
+      tor_fragile_assert();
       return -1;
   }
   return 0;
