@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection_or.c,v 1.169 2005/04/23 14:26:02 nickm Exp $ */
-const char connection_or_c_id[] = "$Id: connection_or.c,v 1.169 2005/04/23 14:26:02 nickm Exp $";
+/* $Id: connection_or.c,v 1.172 2005/04/26 18:52:15 arma Exp $ */
+const char connection_or_c_id[] = "$Id: connection_or.c,v 1.172 2005/04/26 18:52:15 arma Exp $";
 
 /**
  * \file connection_or.c
@@ -148,9 +148,7 @@ int connection_or_finished_flushing(connection_t *conn) {
       break;
     default:
       log_fn(LOG_WARN,"BUG: called in unexpected state %d.", conn->state);
-#ifdef TOR_FRAGILE
-      tor_assert(0);
-#endif
+      tor_fragile_assert();
       return -1;
   }
   return 0;
@@ -171,11 +169,35 @@ int connection_or_finished_connecting(connection_t *conn)
     char buf[1024];
     char addrbuf[INET_NTOA_BUF_LEN];
     struct in_addr in;
+    const char *authenticator = get_options()->HttpsProxyAuthenticator;
 
     in.s_addr = htonl(conn->addr);
     tor_inet_ntoa(&in, addrbuf, sizeof(addrbuf));
-    tor_snprintf(buf, sizeof(buf), "CONNECT %s:%d HTTP/1.0\r\n\r\n",
-                 addrbuf, conn->port);
+
+    if (authenticator) {
+      /* an authenticator in Basic authentication
+       * is just the string "username:password" */
+      const int authenticator_length = strlen(authenticator);
+      /* The base64_encode function needs a minimum buffer length
+       * of 66 bytes. */
+      const int base64_authenticator_length = (authenticator_length/48+1)*66;
+      char *base64_authenticator = tor_malloc(base64_authenticator_length);
+      if (base64_encode(base64_authenticator, base64_authenticator_length,
+                        authenticator, authenticator_length) < 0) {
+        log_fn(LOG_WARN, "Encoding authenticator failed");
+        base64_authenticator[0] = 0;
+      } else {
+        /* remove extra \n at end of encoding */
+        base64_authenticator[strlen(base64_authenticator) - 1] = 0;
+      }
+      tor_snprintf(buf, sizeof(buf), "CONNECT %s:%d HTTP/1.1\r\n"
+                   "Proxy-Authorization: Basic %s\r\n\r\n", addrbuf,
+                   conn->port, base64_authenticator);
+      tor_free(base64_authenticator);
+    } else {
+      tor_snprintf(buf, sizeof(buf), "CONNECT %s:%d HTTP/1.0\r\n\r\n",
+                   addrbuf, conn->port);
+    }
     connection_write_to_buf(buf, strlen(buf), conn);
     conn->state = OR_CONN_STATE_PROXY_FLUSHING;
     return 0;
@@ -440,8 +462,6 @@ connection_tls_finish_handshake(connection_t *conn) {
   int severity = (authdir_mode(options) || !server_mode(options))
                  ? LOG_WARN : LOG_INFO;
 
-  conn->state = OR_CONN_STATE_OPEN;
-  connection_watch_events(conn, EV_READ);
   log_fn(LOG_DEBUG,"tls handshake done. verifying.");
   check_no_tls_errors();
   if (! tor_tls_peer_has_cert(conn->tls)) {
@@ -542,8 +562,9 @@ connection_tls_finish_handshake(connection_t *conn) {
   }
 
   directory_set_dirty();
+  conn->state = OR_CONN_STATE_OPEN;
+  connection_watch_events(conn, EV_READ);
   circuit_n_conn_done(conn, 1); /* send the pending creates, if any. */
-  /* Note the success */
   rep_hist_note_connect_succeeded(conn->identity_digest, time(NULL));
   control_event_or_conn_status(conn, OR_CONN_EVENT_CONNECTED);
   return 0;
