@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: directory.c,v 1.223 2005/04/07 21:09:19 arma Exp $ */
-const char directory_c_id[] = "$Id: directory.c,v 1.223 2005/04/07 21:09:19 arma Exp $";
+/* $Id: directory.c,v 1.224 2005/04/26 18:52:15 arma Exp $ */
+const char directory_c_id[] = "$Id: directory.c,v 1.224 2005/04/26 18:52:15 arma Exp $";
 
 #include "or.h"
 
@@ -1126,9 +1126,7 @@ int connection_dir_finished_flushing(connection_t *conn) {
       return 0;
     default:
       log_fn(LOG_WARN,"Bug: called in unexpected state %d.", conn->state);
-#ifdef TOR_FRAGILE
-      tor_assert(0);
-#endif
+      tor_fragile_assert();
       return -1;
   }
   return 0;
