@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dns.c,v 1.147 2005/04/06 20:25:21 arma Exp $ */
-const char dns_c_id[] = "$Id: dns.c,v 1.147 2005/04/06 20:25:21 arma Exp $";
+/* $Id: dns.c,v 1.150 2005/04/26 20:05:15 arma Exp $ */
+const char dns_c_id[] = "$Id: dns.c,v 1.150 2005/04/26 20:05:15 arma Exp $";
 
 /**
  * \file dns.c
@@ -136,18 +136,15 @@ static void purge_expired_resolves(uint32_t now) {
    */
   while (oldest_cached_resolve && (oldest_cached_resolve->expire < now)) {
     resolve = oldest_cached_resolve;
-    log(LOG_DEBUG,"Forgetting old cached resolve (expires %lu)", (unsigned long)resolve->expire);
+    log(LOG_DEBUG,"Forgetting old cached resolve (address %s, expires %lu)",
+        resolve->address, (unsigned long)resolve->expire);
     if (resolve->state == CACHE_STATE_PENDING) {
-      log_fn(LOG_WARN,"Bug: Expiring a dns resolve that's still pending. Forgot to cull it?");
-#ifdef TOR_FRAGILE
-      tor_assert(0);
-#endif
+      log_fn(LOG_WARN,"Bug: Expiring a dns resolve ('%s') that's still pending. Forgot to cull it?", resolve->address);
+      tor_fragile_assert();
     }
     if (resolve->pending_connections) {
       log_fn(LOG_WARN, "Closing pending connections on expiring DNS resolve!");
-#ifdef TOR_FRAGILE
-      tor_assert(0);
-#endif
+      tor_fragile_assert();
       while (resolve->pending_connections) {
         pend = resolve->pending_connections;
         resolve->pending_connections = pend->next;
@@ -439,9 +436,7 @@ void dns_cancel_pending_resolve(char *address) {
   if (!resolve->pending_connections) {
     /* XXX this should never trigger, but sometimes it does */
     log_fn(LOG_WARN,"Bug: Address '%s' is pending but has no pending connections!", address);
-#ifdef TOR_FRAGILE
-    tor_assert(0);
-#endif
+    tor_fragile_assert();
     return;
   }
   tor_assert(resolve->pending_connections);
@@ -637,10 +632,19 @@ int connection_dns_process_inbuf(connection_t *conn) {
   tor_assert(conn->type == CONN_TYPE_DNSWORKER);
 
   if (conn->state != DNSWORKER_STATE_BUSY && buf_datalen(conn->inbuf)) {
-    log_fn(LOG_WARN,"Bug: read data (%d bytes) from an idle dns worker.  Please report.", (int)buf_datalen(conn->inbuf));
-#ifdef TOR_FRAGILE
-    tor_assert(0);
-#endif
+    log_fn(LOG_WARN,"Bug: read data (%d bytes) from an idle dns worker (fd %d, address '%s'). Please report.",
+           (int)buf_datalen(conn->inbuf), conn->s, conn->address);
+    tor_fragile_assert();
+
+    /* Pull it off the buffer anyway, or it will just stay there.
+     * Keep pulling things off because sometimes we get several
+     * answers at once (!). */
+    while (buf_datalen(conn->inbuf)) {
+      connection_fetch_from_buf(&success,1,conn);
+      connection_fetch_from_buf((char *)&addr,sizeof(uint32_t),conn);
+      log_fn(LOG_WARN,"Discarding idle dns answer (success %d, addr %d.)",
+             success, addr);
+    }
     return 0;
   }
   if (buf_datalen(conn->inbuf) < 5) /* entire answer available? */
