@@ -2,7 +2,7 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: or.h,v 1.594 2005/04/21 10:40:47 arma Exp $ */
+/* $Id: or.h,v 1.596 2005/04/26 18:52:16 arma Exp $ */
 
 /**
  * \file or.h
@@ -12,7 +12,7 @@
 
 #ifndef __OR_H
 #define __OR_H
-#define OR_H_ID "$Id: or.h,v 1.594 2005/04/21 10:40:47 arma Exp $"
+#define OR_H_ID "$Id: or.h,v 1.596 2005/04/26 18:52:16 arma Exp $"
 
 #include "orconfig.h"
 #ifdef MS_WINDOWS
@@ -146,10 +146,6 @@
 #define cell_t tor_cell_t
 #endif
 
-/** Define this if you want Tor to crash when any problem comes up,
- * so you can get a coredump and track things down. */
-#undef TOR_FRAGILE
-
 #define DEFAULT_BANDWIDTH_OP (1024 * 1000)
 #define MAX_NICKNAME_LEN 19
 /* Hex digest plus dollar sign. */
@@ -1059,6 +1055,7 @@ typedef struct {
   char *HttpsProxy; /**< hostname[:port] to use as https proxy, if any */
   uint32_t HttpsProxyAddr; /**< Parsed IPv4 addr for https proxy, if any */
   uint16_t HttpsProxyPort; /**< Parsed port for https proxy, if any */
+  char *HttpsProxyAuthenticator; /** username:password string, if any */
 
   struct config_line_t *DirServers; /**< List of configuration lines
                                      * for directory servers. */
