@@ -2,8 +2,8 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: router.c,v 1.167 2005/04/02 08:55:31 arma Exp $ */
-const char router_c_id[] = "$Id: router.c,v 1.167 2005/04/02 08:55:31 arma Exp $";
+/* $Id: router.c,v 1.168 2005/04/21 10:40:48 arma Exp $ */
+const char router_c_id[] = "$Id: router.c,v 1.168 2005/04/21 10:40:48 arma Exp $";
 
 #include "or.h"
 
@@ -472,18 +472,23 @@ int proxy_mode(or_options_t *options) {
   return (options->SocksPort != 0 || options->SocksBindAddress);
 }
 
-/** Decide if we're a publishable server or just a client. We are a server if:
+/** Decide if we're a publishable server. We are a publishable server if:
+ * - We don't have the ClientOnly option set
+ * and
+ * - We don't have the NoPublish option set
+ * and
+ * - We have ORPort set
+ * and
+ * - We believe we are reachable from the outside; or
  * - We have the AuthoritativeDirectory option set.
- * or
- * - We don't have the ClientOnly option set; and
- * - We have ORPort set; and
- * - We believe we are reachable from the outside.
  */
 static int decide_if_publishable_server(time_t now) {
   or_options_t *options = get_options();
 
   if (options->ClientOnly)
     return 0;
+  if (options->NoPublish)
+    return 0;
   if (!server_mode(options))
     return 0;
   if (options->AuthoritativeDir)
