@@ -1,9 +1,9 @@
 /* Copyright 2001 Matej Pfajfar.
  * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerparse.c,v 1.104 2005/03/31 20:52:43 arma Exp $ */
-const char routerparse_c_id[] = "$Id: routerparse.c,v 1.104 2005/03/31 20:52:43 arma Exp $";
+/* $Id: routerparse.c,v 1.106 2005/04/02 08:30:40 arma Exp $ */
+const char routerparse_c_id[] = "$Id: routerparse.c,v 1.106 2005/04/02 08:30:40 arma Exp $";
 
 /**
  * \file routerparse.c
@@ -724,7 +724,7 @@ static int check_directory_signature(const char *digest,
     _pkey = pkey;
   }
   if (!_pkey) {
-    log_fn(LOG_WARN, "Found directory in old (before 0.0.9pre3) format--rejecting.");
+    log_fn(LOG_WARN, "Obsolete directory format (dir signing key not present) or signing key not trusted--rejecting.");
     return -1;
   }
 
